==> liquid-xp-52.0.0.jar: src/immibis/lxp/LiquidXPMod.java <==
package immibis.lxp;

import immibis.cobaltite.AssignedBlock;
import immibis.cobaltite.AssignedItem;
import immibis.cobaltite.CobaltiteMod;
import immibis.cobaltite.CobaltiteMod.RegisteredTile;
import immibis.cobaltite.Configurable;
import immibis.cobaltite.ModBase;
import immibis.cobaltite.TileGUI;
import immibis.core.api.FMLModInfo;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.liquids.LiquidContainerData;
import net.minecraftforge.liquids.LiquidContainerRegistry;
import net.minecraftforge.liquids.LiquidStack;
import cpw.mods.fml.client.TextureFXManager;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.Init;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.Mod.PreInit;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.registry.GameRegistry;

@Mod(modid="LiquidXP", name = "Liquid XP", version = "52.0.0", dependencies="required-after:ImmibisCore",
		acceptedMinecraftVersions="1.4.6,1.4.7") // update FIELD_NAME_recentlyHit below with obfuscated EntityLiving.recentlyHit name
@NetworkMod(clientSideRequired=true, serverSideRequired=false)
@CobaltiteMod(
	textures = {
		"/immibis/lxp/world.png",
		"/immibis/lxp/imprinter.png",
		"/immibis/lxp/enchanter.png",
		"/immibis/lxp/combiner.png"
	},
	tiles = {
		@RegisteredTile(id="immibis.lxp.collector", tile=CollectorTile.class),
		@RegisteredTile(id="immibis.lxp.absorber", tile=AbsorberTile.class),
		@RegisteredTile(id="immibis.lxp.imprinter", tile=ImprinterTile.class),
		@RegisteredTile(id="immibis.lxp.catalyzer", tile=CatalyzerTile.class),
		@RegisteredTile(id="immibis.lxp.enchanter", tile=EnchanterTile.class),
		@RegisteredTile(id="immibis.lxp.combiner", tile=CombinerTile.class)
	}
)
@FMLModInfo(
	description="Liquid XP and related machines.",
	url="http://www.minecraftforum.net/topic/1001131-110-immibiss-mods-smp/",
	authors="immibis"
)
public class LiquidXPMod extends ModBase {
	public static final String FIELD_NAME_recentlyHit = "bc";
	
	@AssignedItem(id="liquid")
	public static LiquidItem liquid;
	
	@AssignedItem(id="bucket")
	public static BucketItem bucket;
	
	@AssignedItem(id="medallion")
	public static MedallionItem medallion;
	
	@AssignedBlock(id="machine", item=LXPMachineItem.class)
	public static LXPMachineBlock machines;
	
	@TileGUI(container=ImprinterContainer.class, gui=ImprinterGUI.class)
	public static final int GUI_IMPRINTER = 0;
	
	@TileGUI(container=EnchanterContainer.class, gui=EnchanterGUI.class)
	public static final int GUI_ENCHANTER = 1;
	
	@TileGUI(container=CombinerContainer.class, gui=CombinerGUI.class)
	public static final int GUI_COMBINER = 2;
	
	@Instance("LiquidXP")
	public static LiquidXPMod instance;

	@Configurable("onlyEnchantBooks")
	public static boolean ONLY_ENCHANT_BOOKS = true;
	
	public static final double XP_TO_MB = 100; // xp * XP_TO_MB = mb
	public static final double MB_TO_XP = 1.0/XP_TO_MB; // mb * MB_TO_XP = xp

	@Override
	protected void initBlocksAndItems() throws Exception {
		TextureFXManager.instance().addAnimation(new LiquidTextureFX());
		//TextureFXManager.instance().addAnimation(new MedallionTextureFX());
		
		LiquidContainerRegistry.registerLiquid(new LiquidContainerData(new LiquidStack(liquid, 1000), new ItemStack(bucket), new ItemStack(Item.bucketEmpty)));
	}
	
	@Override
	protected void addRecipes() throws Exception {
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_ABSORBER),
			"S_S",
			"SUS",
			"SSS",
			'S', Block.stone,
			'_', Block.pressurePlateStone,
			'U', Item.bucketEmpty
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_CATALYZER),
			"SBS",
			"/D/",
			"D/D",
			'S', Block.stone,
			'D', Item.diamond,
			'/', Item.blazeRod,
			'B', bucket
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_COLLECTOR),
			"SDS",
			"SBS",
			"SSS",
			'S', Block.stone,
			'B', bucket,
			'D', Item.diamond
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_COMBINER),
			"BSB",
			"AFA",
			"SBS",
			'S', Block.stone,
			'B', bucket,
			'F', Block.stoneOvenIdle,
			'A', Block.anvil
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_ENCHANTER),
			"BDB",
			"OEO",
			"OOO",
			'B', bucket,
			'E', Block.enchantmentTable,
			'D', Item.diamond,
			'O', Block.obsidian
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_IMPRINTER),
			"SMS",
			"BPB",
			"SSS",
			'S', Block.stone,
			'B', Block.bookShelf,
			'P', Block.pistonBase,
			'M', new ItemStack(medallion, 1, 0)
			);
		GameRegistry.addRecipe(new ItemStack(medallion, 1, 0),
			" I ",
			"IBI",
			" I ",
			'I', Item.ingotIron,
			'B', bucket
			);
	}
	
	public static int xpBarCap(int level) {
		return level >= 30 ? 62 + (level - 30) * 7 : (level >= 15 ? 17 + (level - 15) * 3 : 17);
	}
	
	public static int levelToXP(int level) {
		int n = 0;
		for(int k = 0; k < level; k++) // this is not off by one
			n += xpBarCap(k);
		return n;
	}
	
	public static int xpToLevel(int experienceTotal) {
		for(int level = 0; ; level++) {
			experienceTotal -= xpBarCap(level);
			if(experienceTotal < 0)
				return level; // this is not off by one
		}
	}
	
	@Init public void init(FMLInitializationEvent evt) {super._init(evt);}
	@PreInit public void preinit(FMLPreInitializationEvent evt) {super._preinit(evt);}
}

==> liquid-xp-52.0.1.jar: src/immibis/lxp/LiquidXPMod.java <==
package immibis.lxp;

import immibis.cobaltite.AssignedBlock;
import immibis.cobaltite.AssignedItem;
import immibis.cobaltite.CobaltiteMod;
import immibis.cobaltite.CobaltiteMod.RegisteredTile;
import immibis.cobaltite.Configurable;
import immibis.cobaltite.ModBase;
import immibis.cobaltite.TileGUI;
import immibis.core.api.FMLModInfo;
import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.liquids.LiquidContainerData;
import net.minecraftforge.liquids.LiquidContainerRegistry;
import net.minecraftforge.liquids.LiquidStack;
import cpw.mods.fml.client.TextureFXManager;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.Init;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.Mod.PreInit;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.registry.GameRegistry;

@Mod(modid="LiquidXP", name = "Liquid XP", version = "52.0.1", dependencies="required-after:ImmibisCore",
		acceptedMinecraftVersions="1.4.6,1.4.7") // update FIELD_NAME_recentlyHit below with obfuscated EntityLiving.recentlyHit name
@NetworkMod(clientSideRequired=true, serverSideRequired=false)
@CobaltiteMod(
	textures = {
		"/immibis/lxp/world.png",
		"/immibis/lxp/imprinter.png",
		"/immibis/lxp/enchanter.png",
		"/immibis/lxp/combiner.png"
	},
	tiles = {
		@RegisteredTile(id="immibis.lxp.collector", tile=CollectorTile.class),
		@RegisteredTile(id="immibis.lxp.absorber", tile=AbsorberTile.class),
		@RegisteredTile(id="immibis.lxp.imprinter", tile=ImprinterTile.class),
		@RegisteredTile(id="immibis.lxp.catalyzer", tile=CatalyzerTile.class),
		@RegisteredTile(id="immibis.lxp.enchanter", tile=EnchanterTile.class),
		@RegisteredTile(id="immibis.lxp.combiner", tile=CombinerTile.class)
	}
)
@FMLModInfo(
	description="Liquid XP and related machines.",
	url="http://www.minecraftforum.net/topic/1001131-110-immibiss-mods-smp/",
	authors="immibis"
)
public class LiquidXPMod extends ModBase {
	public static final String FIELD_NAME_recentlyHit = "bk";
	
	@AssignedItem(id="liquid")
	public static LiquidItem liquid;
	
	@AssignedItem(id="bucket")
	public static BucketItem bucket;
	
	@AssignedItem(id="medallion")
	public static MedallionItem medallion;
	
	@AssignedBlock(id="machine", item=LXPMachineItem.class)
	public static LXPMachineBlock machines;
	
	@TileGUI(container=ImprinterContainer.class, gui=ImprinterGUI.class)
	public static final int GUI_IMPRINTER = 0;
	
	@TileGUI(container=EnchanterContainer.class, gui=EnchanterGUI.class)
	public static final int GUI_ENCHANTER = 1;
	
	@TileGUI(container=CombinerContainer.class, gui=CombinerGUI.class)
	public static final int GUI_COMBINER = 2;
	
	@Instance("LiquidXP")
	public static LiquidXPMod instance;

	@Configurable("onlyEnchantBooks")
	public static boolean ONLY_ENCHANT_BOOKS = true;
	
	public static final double XP_TO_MB = 100; // xp * XP_TO_MB = mb
	public static final double MB_TO_XP = 1.0/XP_TO_MB; // mb * MB_TO_XP = xp

	@Override
	protected void initBlocksAndItems() throws Exception {
		TextureFXManager.instance().addAnimation(new LiquidTextureFX());
		//TextureFXManager.instance().addAnimation(new MedallionTextureFX());
		
		LiquidContainerRegistry.registerLiquid(new LiquidContainerData(new LiquidStack(liquid, 1000), new ItemStack(bucket), new ItemStack(Item.bucketEmpty)));
	}
	
	@Override
	protected void addRecipes() throws Exception {
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_ABSORBER),
			"S_S",
			"SUS",
			"SSS",
			'S', Block.stone,
			'_', Block.pressurePlateStone,
			'U', Item.bucketEmpty
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_CATALYZER),
			"SBS",
			"/D/",
			"D/D",
			'S', Block.stone,
			'D', Item.diamond,
			'/', Item.blazeRod,
			'B', bucket
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_COLLECTOR),
			"SDS",
			"SBS",
			"SSS",
			'S', Block.stone,
			'B', bucket,
			'D', Item.diamond
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_COMBINER),
			"BSB",
			"AFA",
			"SBS",
			'S', Block.stone,
			'B', bucket,
			'F', Block.stoneOvenIdle,
			'A', Block.anvil
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_ENCHANTER),
			"BDB",
			"OEO",
			"OOO",
			'B', bucket,
			'E', Block.enchantmentTable,
			'D', Item.diamond,
			'O', Block.obsidian
			);
		GameRegistry.addRecipe(new ItemStack(machines, 1, LXPMachineItem.META_IMPRINTER),
			"SMS",
			"BPB",
			"SSS",
			'S', Block.stone,
			'B', Block.bookShelf,
			'P', Block.pistonBase,
			'M', new ItemStack(medallion, 1, 0)
			);
		GameRegistry.addRecipe(new ItemStack(medallion, 1, 0),
			" I ",
			"IBI",
			" I ",
			'I', Item.ingotIron,
			'B', bucket
			);
	}
	
	public static int xpBarCap(int level) {
		return level >= 30 ? 62 + (level - 30) * 7 : (level >= 15 ? 17 + (level - 15) * 3 : 17);
	}
	
	public static int levelToXP(int level) {
		int n = 0;
		for(int k = 0; k < level; k++) // this is not off by one
			n += xpBarCap(k);
		return n;
	}
	
	public static int xpToLevel(int experienceTotal) {
		for(int level = 0; ; level++) {
			experienceTotal -= xpBarCap(level);
			if(experienceTotal < 0)
				return level; // this is not off by one
		}
	}
	
	@Init public void init(FMLInitializationEvent evt) {super._init(evt);}
	@PreInit public void preinit(FMLPreInitializationEvent evt) {super._preinit(evt);}
}
